@@ -1584,17 +1584,19 @@ void CmdFigure(int code)
                 fprintRTF("%d about here]", getCounter("endfloatfigure"));
             }
         } else if (g_latex_figures) {
-            char *caption, *label;
+            char *caption, *new_caption, *label;
 
             caption = ExtractAndRemoveTag("\\caption", figure_contents);
             label = ExtractAndRemoveTag("\\label", figure_contents);
 
             PrepareDisplayedBitmap("figure");
-            WriteLatexAsBitmap("\\begin{figure}", figure_contents, "\\end{figure}");
+            WriteLatexAsBitmapOrEPS("\\begin{figure}", figure_contents, "\\end{figure}", BITMAP);
             FinishDisplayedBitmap();
-            ConvertString(caption);
+            new_caption = strdup_together3("\\caption{", caption, "}");
+            ConvertString(new_caption);
             safe_free(label);
             safe_free(caption);
+            safe_free(new_caption);
         } else {
             startParagraph("figure", PARAGRAPH_GENERIC);
             ConvertString(figure_contents);
